
local addonName, addonData = ...

addonData.PlayerNames = {}

-- Color an individual Presence Name and return the value to be added to the table
addonData.ColorPresenceName = function(self,presenceID,dispName)

	-- If we don't seem to have a presence ID then just display the default name
	if not presenceID then
		return dispName
	end

	-- Get details of the active toon
	local toonInfo1, toonName, client, realmName, realmID, faction, race, class, toonInfo9, zoneName, level, 
			gameText, broadcastText, broadcastTime, toonInfo15, presenceID = BNGetToonInfo(presenceID)

	-- Work out the colors for the active toon's class
	local classToken = self.ClassNames[class]
	local color = classToken and RAID_CLASS_COLORS[classToken]
	local colorHex = self:RGB_TO_HEX(color,FRIENDS_BNET_NAME_COLOR)

	-- Now that we have a hex version of the color choose what to display
	-- If we don't know the toon or realm name yet then don't display them
	if colorHex then
		if toonName and toonName ~= "" and self.showToonName then
			if realmName and realmName ~= "" and self.showRealmName then
				return string.format("|cff%s%s (%s-%s)|r",colorHex,dispName,toonName,realmName)
			else
				return string.format("|cff%s%s (%s)|r",colorHex,dispName,toonName)
			end
		else
			return string.format("|cff%s%s|r",colorHex,dispName)
		end
	end
end

-- Rebuild a list of your active friends current toons whenever they come online or you enter the world
addonData.BuildPresenceTable = function(self)

	-- Initialise the player name table
	self.PlayerNames = self.PlayerNames or {}
	wipe(self.PlayerNames)
	
	-- Loop through your battle net friends list
	for i = 1, BNGetNumFriends() do
		local presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, isOnline, lastOnline, 
				isAFK, isDND, messageText, noteText, isRIDFriend, broadcastTime, canSoR = BNGetFriendInfo(i)

		-- if there is a toon online and in wow then process it
		if isOnline and toonID and client == BNET_CLIENT_WOW then
			local dispName = presenceName
			if isBattleTagPresence and battleTag and self.showBattleTag then
				dispName = battleTag
			end
			self.PlayerNames[presenceID] = self:ColorPresenceName(presenceID,dispName)
		end
	end

end

-- Customize the selected chat frame
addonData.CustomizeChatFrame = function(self,chatFrame)
	self[chatFrame] = self[chatFrame] or {}
	if ( not self[chatFrame].AddMessage ) then
		self[chatFrame].AddMessage = chatFrame.AddMessage
		chatFrame.AddMessage = self.AddMessage
	end
end

-- Customize the add message function with our own changes
addonData.AddMessage = function(chatFrame, message, ...)

	if type(message) == "string" then

		local bnNameFormat = "%s"
		local playerBNFormat = "|HBNplayer:%s|h" .. bnNameFormat .. "%s|h"
		local playerBNPattern = "|HBNplayer:(.-)|h%[(|Kf(%d+).-)%](.*)|h"

		-- if the iiLinks addon is in use we need to negate what it does
		if IsAddOnLoaded("iiLinks") then
			if not chatFrame.isTemporary then
				if not string.match(message,"BN_INLINE_TOAST_ALERT") then
					message = message:gsub( "|HBNplayer:(.-)|h(.-)|h", "|HBNplayer:%1|h%[%2%]|h" )
				end
			end
		end		

		-- Substitute the plain name with the colored version of the name
		local presenceData, presenceName, presenceID, presenceExtra = string.match(message, playerBNPattern)
		if presenceData or presenceName then
			if presenceID and not ( addonData.PlayerNames[presenceID] ) then
				addonData.PlayerNames[presenceID] = addonData:ColorPresenceName(presenceID,presenceName)
			end
			local link = string.format(playerBNFormat, presenceData, addonData.PlayerNames[presenceID], presenceExtra or "")
			message = string.gsub(message, playerBNPattern, link)
		end
	end

	addonData[chatFrame].AddMessage(chatFrame,message, ...)

end

